﻿using Battleship.Exceptions;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Battleship.Generics
{

    internal class SetGeneric<T>
        where T : class
    {
        private readonly List<T?> _places;
        public int Count => _places.Count;
        private readonly int _maxCount;
        public SetGeneric(int count)
        {
            _maxCount = count;
            _places = new List<T?>(count);
        }
       
        public void Insert(T ship)
        {
            if(_places.Count == _maxCount)
            {
                throw new StorageOverflowException(_maxCount);
            }
            Insert(ship, 0);
        }
        
        public void Insert(T ship, int position)
        {
            if(_places.Count == _maxCount)
                throw new StorageOverflowException(_maxCount);
            if (position < 0 || position >= _maxCount)
                throw new ShipNotFoundException("Impossible to insert");
            _places.Insert(position, ship);

        }
        public void Remove(int position)
        {
            if (position < 0 || position > _maxCount || position >= Count)
                throw new ShipNotFoundException(position);

            _places.RemoveAt(position);
        }
        public T? this[int position]
        {
            get
            {
                if (position < 0 || position >= Count)
                    return null;
                return _places[position];

            }
            set
            {
                if (position < 0 || position > _maxCount || Count == _maxCount)
                    return;
                _places[position] = value;
            }

        }
        public IEnumerable<T> GetShips(int? maxShips = null)
        {
            for (int i = 0; i < _places.Count; ++i)
            {
                yield return _places[i];
                if (maxShips.HasValue && i == maxShips.Value)
                {
                    yield break;
                }
            }
        }
    }
}

